use ExtUtils::MakeMaker;
use IO::File;
use Getopt::Long;

# See lib/ExtUtils/MakeMaker.pm for details of how to influence
# the contents of the Makefile that is written.

my $CC = "";
my $TMPC = "test.c";
my $TMPO = "test";
my $TMPLOG = "config.log";

my $MJPEGROOT = "";
my $MJPEGINC  = "";
my $MJPEGLIB  = "";
my $help;

GetOptions("with-mjpeginc=s" => \$MJPEGINC,
           "with-mjpeglib=s" => \$MJPEGLIB,
           "with-mjpeg=s" => \$MJPEGROOT,
           "help" => \$help);

if ($help)
{
  print "Usage: Makefile.PL [options]
Directory and file names:
  --with-mjpeg=PREFIX   Prefix for mjpegutils-related files.
                        Headers are:   PREFIX/include/mjpegutils
                        Libraries are: PREFIX/lib/libmjpegutils.a
  --with-mjpeginc=DIR   location where mpegutils headers are found.
  --with-mjpeglib=DIR   location where libmjpegutils.a is found.

  If you do not specify a location for the mpegutils files, this script
  will attempt to autodetect.  This means checking /usr and /usr/local.
";

  exit(0);  
}

WriteMakefile(
    'NAME'		=> 'YUV4MPEG',
    'VERSION_FROM'	=> 'YUV4MPEG.pm',
    'PREREQ_PM'		=> {},
    ($] >= 5.005 ?
      (ABSTRACT_FROM => 'YUV4MPEG.pm',
       AUTHOR     => 'Brett Kosinski <brettk@frodo.dyn.gno.org>') : ()),
    'CONFIGURE'         => \&configure,
    'OBJECT'		=> '$(O_FILES)',
    'clean'             => {FILES => $TMPLOG}
);

sub configure
{
  my ($lib, $inc, $def, @typemaps);
  my $code;

  STDOUT->autoflush(1);
  unlink $TMPLOG;

  print "Checking for gcc... ";

  $CC = "gcc";

  if (cc_check())
    {
      print " yes\n";
    }
  else
    {
      print " no\n";
      print "Checking for cc... ";

      $CC = "cc";

      if (cc_check())
        {
          print " yes\n";
        }
      else
        {
          print " no\n";

          die "Unable to determine which compiler to use!\n";
        }
    }

  print "Checking for PRId64... ";

  $code = '
int main()
{
  PRId64;

  return 0;
}
';

  if (cc_check("", "#include <inttypes.h>\n" . $code))
    {
      $def .= "-DHAVE_INTTYPES_H ";

      print "inttypes.h\n";
    }
  elsif (cc_check("", "#include <stdint.h>\n" . $code))
    {
      $def .= "-DHAVE_STDINT_H ";

      print "stdint.h\n";
    }
  else 
    {
      print "no\n";

      die "Could not find PRId64 anywhere!\n";
    }

  print "Checking for stdbool.h... ";

  $code = '
#include <stdbool.h>

int main()
{
  return 0;
}
';

  if (cc_check("", $code))
    {
      $def .= "-DHAVE_STDBOOL_H ";

      print "yes\n";
    }
  else
    {
      print "no\n";
    }

  print "Checking for mjpegutils library... ";

  if ($MJPEGLIB ne "")
    {
      if (cc_check("-L$MJPEGLIB -lmjpegutils"))
        {
          $lib .= "-L$MJPEGLIB -lmjpegutils";
          print "$MJPEGLIB\n";
        }
      else
        {
          print "no\n";

          die "Couldn't find mjpegutils library!\n";
        }
    }
  elsif ($MJPEGROOT ne "")
    {
      if (cc_check("-L$MJPEGROOT/lib -lmjpegutils"))
        {
          $lib .= "-L$MJPEGROOT/lib -lmjpegutils";
          print "$MJPEGROOT/lib\n";
        }
      else
        {
          print "no\n";

          die "Couldn't find mjpegutils library!\n";
        }
    }
  elsif (cc_check("-L/usr/local/lib -lmjpegutils"))
    {
      $lib .= "-L/usr/local/lib -lmjpegutils";

      print "/usr/local/lib\n";
    }
  elsif (cc_check("-L/usr/lib -lmjpegutils"))
    {
      $lib .= "-L/usr/lib -lmjpegutils";
      print "/usr/lib\n";
    }
  else
    {
      print "no\n";

      die "Couldn't find mjpegutils library!\n";
    }

  print "Checking for YUV4MPEG header... ";

  $code = '
#include <yuv4mpeg.h>

int main()
{
  return 0;
}
';

  if ($MJPEGINC ne "")
    {
      if (cc_check("-I$MJPEGINC $def", $code))
        {
          $inc .= "-I$MJPEGINC";
          print "$MJPEGINC\n";
        }
      else
        {
          print "no\n";

          die "Couldn't find YUV4MPEG headers!\n";
        }
    }
  elsif ($MJPEGROOT ne "")
    {
      if (cc_check("-I$MJPEGROOT/include/mjpegtools $def", $code))
        {
          $inc .= "-I$MJPEGROOT/include/mjpegtools";
          print "$MJPEGROOT/include/mjpegtools\n";
        }
      else
        {
          print "no\n";

          die "Couldn't find YUV4MPEG headers!\n";
        }
    }
  elsif (cc_check("-I/usr/local/include/mjpegtools $def", $code))
    {
      $inc .= "-I/usr/local/include/mjpegtools";

      print "/usr/local/include/mjpegtools\n";
    }
  elsif (cc_check("-I/usr/include/mjpegtools $def", $code))
    {
      $inc .= "-I/usr/include/mjpegtools";

      print "/usr/include/mjpegtools\n";
    }
  else
    {
      print "no\n";

      die "Couldn't find YUV4MPEG headers!\n";
    }

  print "Checking for y4m_fi_xtags... ";

  $code = '
#include <yuv4mpeg.h>

int main()
{
  y4m_frame_info_t frame;

  y4m_fi_xtags(&frame);

  return 0;
}
';

  if (cc_check("$inc $def", $code, "$lib"))
    {
      print "yes\n";

      $def .= "-DHAVE_Y4M_FI_XTAGS ";
    }
  else
    {
      print "no - compiling without frame tags support.\n";
    }

  print "Checking for y4m_clear_*_info... ";

  $code = '
#include <yuv4mpeg.h>

int main()
{
  y4m_stream_info_t stream;
  y4m_frame_info_t frame;

  y4m_init_stream_info(&stream);
  y4m_init_frame_info(&frame);

  y4m_clear_stream_info(&stream);
  y4m_clear_frame_info(&frame);

  return 0;
}';

  if (cc_check("$inc $def", $code, "$lib"))
    {
      print "yes\n";

      $def .= "-DHAVE_CLEAR_X_INFO ";
    }
  else
    {
      print "no\n";
    }

  my $log = new IO::File ">>$TMPLOG";

  print $log "Compiler settings are:\n";
  print $log "  $lib $inc $def\n";

  close $log;

  print "Checking for PDL... ";

  my $handler = $SIG{__DIE__};

  $SIG{__DIE__} = sub { return; };

  eval
    {
      require PDL::Core::Dev;
      import PDL::Core::Dev;
    };

  $SIG{__DIE__} = $handler;

  if ($@ eq "")
    {
      print "yes\n";

      $def .= " -DHAVE_PDL";
      $inc .= " " . &PDL_INCLUDE;

      push @typemaps, &PDL_TYPEMAP;
    }
  else
    {
      print "no\n";

      push @typemaps, "typemap.pdl";
    }
  
  return { 'LIBS' => $lib,
           'INC'  => $inc,
           'DEFINE' => $def,
           'TYPEMAPS' => \@typemaps};
}

sub cc_check
{
  my ($opts, $code, $opts2) = @_;
  my $file = new IO::File ">$TMPC";
  my $log = new IO::File ">>$TMPLOG";

  if ($code eq "") 
    {
      $code = "
int main()
{
  return 0;
}
";
    }

  print $file $code;
  close $file;

  my $cmd = "$CC $opts $TMPC -o $TMPO $opts2";

  print $log "Compiling: $cmd\n";
  print $log "----\n";
  print $log $code, "\n";
  print $log "----\n";

  my $cc = new IO::File "$cmd 2>&1|";
  my @result = <$cc>;
  my $ret = -e $TMPO;

  print $log @result;
  print $log "Exit status: $?\n\n";

  close $log;
  close $cc;
  unlink "$TMPC";
  unlink "$TMPO";

  return $ret;
}
